@@ -1,4 +1,4 @@
-// Copyright (C) 2007 MySQL AB
+// Copyright (c) 2004-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License version 2 as published by
@@ -39,6 +39,7 @@ using MySql.Web.Profile;
 using MySql.Web.Common;
 using System.Transactions;
 using System.Text.RegularExpressions;
+using MySql.Web.General;
 
 namespace MySql.Web.Security
 {
@@ -52,7 +53,7 @@ namespace MySql.Web.Security
     /// </para>
     /// </remarks>
     /// <example>
-    /// <code lang="" source="CodeExamples\MembershipCodeExample2.xml"/>
+    /// <code source="CodeExamples/MembershipCodeExample2.xml"/>
     /// </example>
     public sealed class MySQLMembershipProvider : MembershipProvider
     {
@@ -63,7 +64,6 @@ namespace MySql.Web.Security
         private string connectionString;
         private int minRequiredPasswordLength;
         private bool writeExceptionsToEventLog;
-        private string applicationName;
         private bool enablePasswordReset;
         private bool enablePasswordRetrieval;
         private bool requiresQuestionAndAnswer;
@@ -73,7 +73,7 @@ namespace MySql.Web.Security
         private MembershipPasswordFormat passwordFormat;
         private int minRequiredNonAlphanumericCharacters;
         private string passwordStrengthRegularExpression;
-        private int applicationId;
+        private Application app;
 
         /// <summary>
         /// Initializes the MySQL membership provider with the property values specified in the 
@@ -103,12 +103,12 @@ namespace MySql.Web.Security
             }
             base.Initialize(name, config);
 
-            applicationName = GetConfigValue(config["applicationName"], 
+            string applicationName = GetConfigValue(config["applicationName"], 
                 HostingEnvironment.ApplicationVirtualPath);
             maxInvalidPasswordAttempts = Convert.ToInt32(GetConfigValue(config["maxInvalidPasswordAttempts"], "5"));
             passwordAttemptWindow = Convert.ToInt32(GetConfigValue(config["passwordAttemptWindow"], "10"));
             minRequiredNonAlphanumericCharacters =
-                Convert.ToInt32(GetConfigValue(config["minRequiredAlphaNumericCharacters"], "1"));
+                Convert.ToInt32(GetConfigValue(config["minRequiredNonalphanumericCharacters"], "1"));
             minRequiredPasswordLength = Convert.ToInt32(GetConfigValue(config["minRequiredPasswordLength"], "7"));
             passwordStrengthRegularExpression =
                 Convert.ToString(GetConfigValue(config["passwordStrengthRegularExpression"], ""));
@@ -119,6 +119,7 @@ namespace MySql.Web.Security
             requiresUniqueEmail = Convert.ToBoolean(GetConfigValue(config["requiresUniqueEmail"], "True"));
             writeExceptionsToEventLog = Convert.ToBoolean(GetConfigValue(config["writeExceptionsToEventLog"], "True"));
             string temp_format = config["passwordFormat"];
+
             if (temp_format == null)
                 temp_format = "hashed";
             else
@@ -135,8 +136,13 @@ namespace MySql.Web.Security
 
             // if the user is asking for the ability to retrieve hashed passwords, then let
             // them know we can't
-            if (PasswordFormat == MembershipPasswordFormat.Hashed && EnablePasswordRetrieval)
-                throw new ProviderException(Resources.CannotRetrieveHashedPasswords);
+            if (PasswordFormat == MembershipPasswordFormat.Hashed)
+            {
+                if (EnablePasswordRetrieval)
+                    throw new ProviderException(Resources.CannotRetrieveHashedPasswords);
+                if (Runtime.IsMono)
+                    throw new ProviderException(Resources.MonoDoesNotSupportHash);
+            }
 
             ConnectionStringSettings ConnectionStringSettings = ConfigurationManager.ConnectionStrings[
                 config["connectionStringName"]];
@@ -150,23 +156,7 @@ namespace MySql.Web.Security
             // make sure we have the correct schema
             SchemaManager.CheckSchema(connectionString, config);
 
-            try
-            {
-                // now pre-cache the applicationId
-                using (MySqlConnection conn = new MySqlConnection(connectionString))
-                {
-                    conn.Open();
-                    MySqlCommand cmd = new MySqlCommand("SELECT id FROM my_aspnet_Applications WHERE name=@name", conn);
-                    cmd.Parameters.AddWithValue("@name", applicationName);
-                    object appId = cmd.ExecuteScalar();
-                    if (appId != null)
-                        applicationId = Convert.ToInt32(appId);
-                }
-            }
-            catch (Exception ex)
-            {
-                throw new ProviderException(Resources.ErrorInitOfMembershipProvider, ex);
-            }
+            app = new Application(applicationName, base.Description);
         }
 
         private static string GetConfigValue(string configValue, string defaultValue)
@@ -196,12 +186,12 @@ namespace MySql.Web.Security
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
         /// The application name setting is being used.
-        /// <code lang="" source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override string ApplicationName
         {
-            get { return applicationName; }
-            set { applicationName = value; }
+            get { return app.Name; }
+            set { app.Name = value; }
         }
 
         /// <summary>
@@ -213,7 +203,7 @@ namespace MySql.Web.Security
         /// retrieved.</remarks>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override bool EnablePasswordReset
         {
@@ -231,7 +221,7 @@ namespace MySql.Web.Security
         /// set to true then a <see cref="ProviderException"/> is thrown.</remarks>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override bool EnablePasswordRetrieval
         {
@@ -247,7 +237,7 @@ namespace MySql.Web.Security
         /// reset and retrieval; otherwise, false. The default is false.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override bool RequiresQuestionAndAnswer
         {
@@ -262,7 +252,7 @@ namespace MySql.Web.Security
         /// otherwise, false. The default is true.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override bool RequiresUniqueEmail
         {
@@ -277,7 +267,7 @@ namespace MySql.Web.Security
         /// before the membership user is locked out.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override int MaxInvalidPasswordAttempts
         {
@@ -292,7 +282,7 @@ namespace MySql.Web.Security
         /// password-answer attempts are allowed before the membership user is locked out.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override int PasswordAttemptWindow
         {
@@ -306,7 +296,7 @@ namespace MySql.Web.Security
         /// values indicating the format for storing passwords in the data store.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override MembershipPasswordFormat PasswordFormat
         {
@@ -320,7 +310,7 @@ namespace MySql.Web.Security
         /// in a valid password.</value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override int MinRequiredNonAlphanumericCharacters
         {
@@ -333,7 +323,7 @@ namespace MySql.Web.Security
         /// <value>The minimum length required for a password. </value>
         /// <example>
         /// The following example shows the membership element being used in an applications web.config file.
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override int MinRequiredPasswordLength
         {
@@ -353,7 +343,7 @@ namespace MySql.Web.Security
         /// <list>Contains at least one digit.</list>
         /// <list>Contains at least one special (non-alphanumeric) character.</list>
         /// </ul>
-        /// <code source="CodeExamples\MembershipCodeExample1.xml"/>
+        /// <code source="CodeExamples/MembershipCodeExample1.xml"/>
         /// </example>
         public override string PasswordStrengthRegularExpression
         {
@@ -516,6 +506,8 @@ namespace MySql.Web.Security
                 return null;
             }
 
+            ValidateQA(passwordQuestion, passwordAnswer);
+
             // now try to validate the password
             if (!ValidatePassword(password, "password", false))
             {
@@ -542,12 +534,9 @@ namespace MySql.Web.Security
                     connection.Open();
                     transaction = connection.BeginTransaction();
 
-                    // create or fetch a new application id
-                    SchemaManager.CreateOrFetchApplicationId(applicationName, 
-                        ref applicationId, base.Description, connection);
                     // either create a new user or fetch the existing user id
                     int userId = SchemaManager.CreateOrFetchUserId(connection, username, 
-                        applicationId, true);
+                        app.EnsureId(connection), true);
 
                     MySqlCommand cmd = new MySqlCommand(
                         @"INSERT INTO my_aspnet_Membership 
@@ -634,7 +623,7 @@ namespace MySql.Web.Security
 
                     MySqlCommand cmd = new MySqlCommand(
                         String.Format(sql, deleteAllRelatedData ? "u," : ""), conn);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(conn));
                     cmd.Parameters.AddWithValue("@userId", userId);
                     return cmd.ExecuteNonQuery() > 0;
                 }
@@ -683,7 +672,7 @@ namespace MySql.Web.Security
                         ON m.userId=u.id WHERE m.LastActivityDate > @date AND u.applicationId=@appId", 
                         connection);
                     cmd.Parameters.AddWithValue("@date", compareTime);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(connection));
                     return Convert.ToInt32(cmd.ExecuteScalar());
                 }
             }
@@ -730,10 +719,10 @@ namespace MySql.Web.Security
                             throw new MembershipPasswordException(Resources.UserIsLockedOut);
 
                         string password = reader.GetString("Password");
-                        string passwordAnswer = reader.GetString("PasswordAnswer");
+                        string passwordAnswer = reader.GetValue(reader.GetOrdinal("PasswordAnswer")).ToString();
                         string passwordKey = reader.GetString("PasswordKey");
-                        MembershipPasswordFormat format = (MembershipPasswordFormat)
-                            reader.GetInt32(3);
+                        MembershipPasswordFormat format = (MembershipPasswordFormat)reader.GetInt32(3);
+                        reader.Close();
 
                         if (RequiresQuestionAndAnswer &&
                             !(CheckPassword(answer, passwordAnswer, passwordKey, format)))
@@ -769,15 +758,16 @@ namespace MySql.Web.Security
         {
             try
             {
+                int userId = -1;
                 using (MySqlConnection connection = new MySqlConnection(connectionString))
                 {
                     connection.Open();
 
-                    int userId = GetUserId(connection, username);
+                    userId = GetUserId(connection, username);
                     if (-1 == userId) return null;
-
-                    return GetUser(userId, userIsOnline);
                 }
+
+                return GetUser(userId, userIsOnline);
             }
             catch (MySqlException e)
             {
@@ -797,45 +787,47 @@ namespace MySql.Web.Security
         /// </returns>
         public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
         {
+            MySqlTransaction txn = null;
+
             try
             {
-                using (TransactionScope scope = new TransactionScope())
+                using (MySqlConnection connection = new MySqlConnection(connectionString))
                 {
-                    using (MySqlConnection connection = new MySqlConnection(connectionString))
+                    connection.Open();
+
+                    txn = connection.BeginTransaction();
+                    MySqlCommand cmd = new MySqlCommand("", connection);
+                    cmd.Parameters.AddWithValue("@userId", providerUserKey);
+
+                    if (userIsOnline)
                     {
-                        connection.Open();
+                        cmd.CommandText =
+                            @"UPDATE my_aspnet_Users SET lastActivityDate = @date WHERE id=@userId";
+                        cmd.Parameters.AddWithValue("@date", DateTime.Now);
+                        cmd.ExecuteNonQuery();
 
-                        MySqlCommand cmd = new MySqlCommand("", connection);
-                        cmd.Parameters.AddWithValue("@userId", providerUserKey);
-
-                        if (userIsOnline)
-                        {
-                            cmd.CommandText =
-                                @"UPDATE my_aspnet_Users SET lastActivityDate = @date WHERE id=@userId";
-                            cmd.Parameters.AddWithValue("@date", DateTime.Now);
-                            cmd.ExecuteNonQuery();
-
-                            cmd.CommandText = "UPDATE my_aspnet_Membership SET LastActivityDate=@date WHERE userId=@userId";
-                            cmd.ExecuteNonQuery();
-                        }
-
-                        cmd.CommandText = @"SELECT m.*,u.name 
-                        FROM my_aspnet_Membership m JOIN my_aspnet_Users u ON m.userId=u.id 
-                        WHERE u.id=@userId";
-
-                        MembershipUser user;
-                        using (MySqlDataReader reader = cmd.ExecuteReader())
-                        {
-                            if (!reader.Read()) return null;
-                            user = GetUserFromReader(reader);
-                        }
-
-                        return user;
+                        cmd.CommandText = "UPDATE my_aspnet_Membership SET LastActivityDate=@date WHERE userId=@userId";
+                        cmd.ExecuteNonQuery();
                     }
+
+                    cmd.CommandText = @"SELECT m.*,u.name 
+                    FROM my_aspnet_Membership m JOIN my_aspnet_Users u ON m.userId=u.id 
+                    WHERE u.id=@userId";
+
+                    MembershipUser user;
+                    using (MySqlDataReader reader = cmd.ExecuteReader())
+                    {
+                        if (!reader.Read()) return null;
+                        user = GetUserFromReader(reader);
+                    }
+                    txn.Commit();
+                    return user;
                 }
             }
             catch (MySqlException e)
             {
+                if (txn != null)
+                    txn.Rollback();
                 if (WriteExceptionsToEventLog)
                     WriteToEventLog(e, "GetUser(Object, Boolean)");
                 throw new ProviderException(exceptionMessage);
@@ -898,7 +890,7 @@ namespace MySql.Web.Security
                         WHERE m.Email like @email AND u.applicationId=@appId";
                     MySqlCommand cmd = new MySqlCommand(sql, conn);
                     cmd.Parameters.AddWithValue("@email", email);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(conn));
                     return (string)cmd.ExecuteScalar();
                 }
             }
@@ -962,16 +954,18 @@ namespace MySql.Web.Security
                         if (reader.GetBoolean("IsLockedOut"))
                             throw new MembershipPasswordException(Resources.UserIsLockedOut);
 
-                        string passwordAnswer = reader.GetString("PasswordAnswer");
+                        object passwordAnswer = reader.GetValue(reader.GetOrdinal("PasswordAnswer"));
                         passwordKey = reader.GetString("PasswordKey");
                         format = (MembershipPasswordFormat)reader.GetByte("PasswordFormat");
                         reader.Close();
 
-                        if (RequiresQuestionAndAnswer &&
-                            !CheckPassword(answer, passwordAnswer, passwordKey, format))
+                        if (RequiresQuestionAndAnswer)
                         {
-                            UpdateFailureCount(userId, "PasswordAnswer", connection);
-                            throw new MembershipPasswordException(Resources.IncorrectPasswordAnswer);
+                            if (!CheckPassword(answer, (string)passwordAnswer, passwordKey, format))
+                            {
+                                UpdateFailureCount(userId, "PasswordAnswer", connection);
+                                throw new MembershipPasswordException(Resources.IncorrectPasswordAnswer);
+                            }
                         }
                     }
 
@@ -1025,7 +1019,7 @@ namespace MySql.Web.Security
                     cmd.Parameters.AddWithValue("@lastLoginDate", user.LastLoginDate);
                     cmd.Parameters.AddWithValue("@lastActivityDate", user.LastActivityDate);
                     cmd.Parameters.AddWithValue("@name", user.UserName);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(conn));
                     cmd.ExecuteNonQuery();
                 }
             }
@@ -1063,7 +1057,6 @@ namespace MySql.Web.Security
                             Islockedout FROM my_aspnet_Membership WHERE userId=@userId";
                     MySqlCommand cmd = new MySqlCommand(sql, connection);
                     cmd.Parameters.AddWithValue("@userId", userId);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
 
                     using (MySqlDataReader reader = cmd.ExecuteReader(CommandBehavior.SingleRow))
                     {
@@ -1092,7 +1085,6 @@ namespace MySql.Web.Security
                             updateCmd.Parameters.AddWithValue("@lastLoginDate", currentDate);
                             updateCmd.Parameters.AddWithValue("@date", currentDate);
                             updateCmd.Parameters.AddWithValue("@userid", userId);
-                            updateCmd.Parameters.AddWithValue("@appId", applicationId);
                             updateCmd.ExecuteNonQuery();
                         }
                     }
@@ -1146,8 +1138,9 @@ namespace MySql.Web.Security
         private int GetUserId(MySqlConnection connection, string username)
         {
             MySqlCommand cmd = new MySqlCommand(
-                "SELECT id FROM my_aspnet_Users WHERE name LIKE @name", connection);
+                "SELECT id FROM my_aspnet_Users WHERE name LIKE @name AND applicationId=@appId", connection);
             cmd.Parameters.AddWithValue("@name", username);
+            cmd.Parameters.AddWithValue("@appId", app.FetchId(connection));
             object id = cmd.ExecuteScalar();
             if (id == null) return -1;
             return (int)id;
@@ -1208,8 +1201,8 @@ namespace MySql.Web.Security
             if (format == MembershipPasswordFormat.Clear)
                 return encodedPassword;
             else if (format == MembershipPasswordFormat.Encrypted)
-                return Encoding.Unicode.GetString(
-                    DecryptPassword(Convert.FromBase64String(password)));
+                return Encoding.Unicode.GetString(DecryptPassword(
+                    Convert.FromBase64String(password)));
             else if (format == MembershipPasswordFormat.Hashed)
                 throw new ProviderException(Resources.CannotUnencodeHashedPwd);
             else
@@ -1225,6 +1218,18 @@ namespace MySql.Web.Security
             return Convert.ToBase64String(key);
         }
 
+        /// <summary>
+        /// this method is only necessary because early versions of Mono did not support
+        /// the HashAlgorithmType property
+        /// </summary>
+        /// <param name="bytes"></param>
+        /// <returns></returns>
+        private string HashPasswordBytes(byte[] bytes)
+        {
+            HashAlgorithm hash = HashAlgorithm.Create(Membership.HashAlgorithmType);
+            return Convert.ToBase64String(hash.ComputeHash(bytes));
+        }
+
         private string EncodePassword(string password, string passwordKey,
             MembershipPasswordFormat format)
         {
@@ -1241,14 +1246,11 @@ namespace MySql.Web.Security
 
             if (format == MembershipPasswordFormat.Encrypted)
             {
-                byte[] encryptedBytes = EncryptPassword(keyedBytes);
+                byte[] encryptedBytes = EncryptPassword(passwordBytes);
                 return Convert.ToBase64String(encryptedBytes);
             }
             else if (format == MembershipPasswordFormat.Hashed)
-            {
-                HashAlgorithm hash = HashAlgorithm.Create(Membership.HashAlgorithmType);
-                return Convert.ToBase64String(hash.ComputeHash(keyedBytes));
-            }
+                return HashPasswordBytes(keyedBytes);
             else
                 throw new ProviderException(Resources.UnsupportedPasswordFormat);
         }
@@ -1407,7 +1409,7 @@ namespace MySql.Web.Security
                     }
                     sql += " ORDER BY u.id ASC LIMIT {0},{1}";
                     cmd.CommandText = String.Format(sql, pageIndex * pageSize, pageSize);
-                    cmd.Parameters.AddWithValue("@appId", applicationId);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(connection));
                     using (MySqlDataReader reader = cmd.ExecuteReader())
                     {
                         while (reader.Read())
@@ -1427,6 +1429,14 @@ namespace MySql.Web.Security
             }
         }
 
+        private void ValidateQA(string question, string answer)
+        {
+            if (RequiresQuestionAndAnswer && String.IsNullOrEmpty(question))
+                throw new ArgumentException(Resources.PasswordQuestionInvalid);
+            if (RequiresQuestionAndAnswer && String.IsNullOrEmpty(answer))
+                throw new ArgumentException(Resources.PasswordAnswerInvalid);
+        }
+
         private bool ValidatePassword(string password, string argumentName, bool throwExceptions)
         {
             string exceptionString = null;
