@@ -1,4 +1,4 @@
-// Copyright (C) 2007 MySQL AB
+// Copyright (c) 2004-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License version 2 as published by
@@ -39,6 +39,7 @@ using System.Transactions;
 using System.Web.Security;
 using MySql.Web.Common;
 using MySql.Web.Properties;
+using MySql.Web.General;
 
 namespace MySql.Web.Profile
 {
@@ -47,9 +48,8 @@ namespace MySql.Web.Profile
     /// </summary>
     public class MySQLProfileProvider : ProfileProvider
     {
-        private string applicationName;
         private string connectionString;
-        private int applicationId;
+        private Application app;
 
         #region Abstract Members
 
@@ -63,7 +63,6 @@ namespace MySql.Web.Profile
         /// <exception cref="T:System.InvalidOperationException">An attempt is made to call <see cref="M:System.Configuration.Provider.ProviderBase.Initialize(System.String,System.Collections.Specialized.NameValueCollection)"/> on a provider after the provider has already been initialized.</exception>
         public override void Initialize(string name, NameValueCollection config)
         {
-            applicationId = -1;
             if (config == null)
                 throw new ArgumentNullException("config");
 
@@ -79,30 +78,20 @@ namespace MySql.Web.Profile
 
             try
             {
-                applicationName = GetConfigValue(config["applicationName"], HostingEnvironment.ApplicationVirtualPath);
+                string applicationName = GetConfigValue(config["applicationName"], HostingEnvironment.ApplicationVirtualPath);
 
+                connectionString = "";
                 ConnectionStringSettings ConnectionStringSettings = ConfigurationManager.ConnectionStrings[
                     config["connectionStringName"]];
                 if (ConnectionStringSettings != null)
                     connectionString = ConnectionStringSettings.ConnectionString.Trim();
-                else
-                    connectionString = "";
 
                 if (String.IsNullOrEmpty(connectionString)) return;
 
                 // make sure our schema is up to date
                 SchemaManager.CheckSchema(connectionString, config);
 
-                // now pre-cache the applicationId
-                using (MySqlConnection conn = new MySqlConnection(connectionString))
-                {
-                    conn.Open();
-                    MySqlCommand cmd = new MySqlCommand("SELECT id FROM my_aspnet_Applications WHERE name=@name", conn);
-                    cmd.Parameters.AddWithValue("@name", applicationName);
-                    object appIdValue = cmd.ExecuteScalar();
-                    if (appIdValue != null)
-                        applicationId = Convert.ToInt32(appIdValue);
-                }
+                app = new Application(applicationName, base.Description);
             }
             catch (Exception ex)
             {
@@ -140,7 +129,7 @@ namespace MySql.Web.Profile
                     WHERE applicationId=@appId AND 
                     lastActivityDate < @lastActivityDate",
                     c);
-                queryCmd.Parameters.AddWithValue("@appId", applicationId);
+                queryCmd.Parameters.AddWithValue("@appId", app.FetchId(c));
                 queryCmd.Parameters.AddWithValue("@lastActivityDate", userInactiveSinceDate);
                 if (authenticationOption == ProfileAuthenticationOption.Anonymous)
                     queryCmd.CommandText += " AND isAnonymous = 1";
@@ -186,7 +175,7 @@ namespace MySql.Web.Profile
                 MySqlCommand queryCmd = new MySqlCommand(
                     @"SELECT * FROM my_aspnet_Users  
                     WHERE applicationId=@appId AND name = @name", c);
-                queryCmd.Parameters.AddWithValue("@appId", applicationId);
+                queryCmd.Parameters.AddWithValue("@appId", app.FetchId(c));
                 queryCmd.Parameters.Add("@name", MySqlDbType.VarChar);
 
                 MySqlCommand deleteCmd = new MySqlCommand(
@@ -371,7 +360,7 @@ namespace MySql.Web.Profile
                     WHERE applicationId = @appId AND 
                     lastActivityDate < @lastActivityDate",
                     c);
-                queryCmd.Parameters.AddWithValue("@appId", applicationId);
+                queryCmd.Parameters.AddWithValue("@appId", app.FetchId(c));
                 queryCmd.Parameters.AddWithValue("@lastActivityDate", userInactiveSinceDate);
                 if (authenticationOption == ProfileAuthenticationOption.Anonymous)
                     queryCmd.CommandText += " AND isAnonymous = 1";
@@ -381,22 +370,25 @@ namespace MySql.Web.Profile
             }
         }
 
+        /// <summary>
+        /// Gets or sets the name of the currently running application.
+        /// </summary>
+        /// <value></value>
+        /// <returns>A <see cref="T:System.String"/> that contains the application's shortened name, which does not contain a full path or extension, for example, SimpleAppSettings.</returns>
         public override string ApplicationName
         {
-            get { return applicationName; }
-            set { applicationName = value; }
-        }
-
-        public override string Name
-        {
-            get { return "MySQLProfileProvider"; }
-        }
-
-        public override string Description
-        {
-            get { return "MySQL Profile provider"; }
+            get { return app.Name; }
+            set { app.Name = value; }
         }
 
+        /// <summary>
+        /// Returns the collection of settings property values for the specified application instance and settings property group.
+        /// </summary>
+        /// <param name="context">A <see cref="T:System.Configuration.SettingsContext"/> describing the current application use.</param>
+        /// <param name="collection">A <see cref="T:System.Configuration.SettingsPropertyCollection"/> containing the settings property group whose values are to be retrieved.</param>
+        /// <returns>
+        /// A <see cref="T:System.Configuration.SettingsPropertyValueCollection"/> containing the values for the specified settings property group.
+        /// </returns>
         public override SettingsPropertyValueCollection GetPropertyValues(
             SettingsContext context, SettingsPropertyCollection collection)
         {
@@ -408,6 +400,11 @@ namespace MySql.Web.Profile
 
             foreach (SettingsProperty property in collection)
             {
+                if (property.PropertyType.IsPrimitive || property.PropertyType == typeof(string))
+                    property.SerializeAs = SettingsSerializeAs.String;
+                else
+                    property.SerializeAs = SettingsSerializeAs.Xml;
+
                 values.Add(new SettingsPropertyValue(property));
             }
 
@@ -417,19 +414,24 @@ namespace MySql.Web.Profile
             // retrieve encoded profile data from the database
             try
             {
-                MySqlConnection c = new MySqlConnection(connectionString);
-                MySqlCommand cmd = new MySqlCommand(@"SELECT * FROM my_aspnet_Profiles p
-                JOIN my_aspnet_Users u ON u.id = p.userId
-                WHERE u.applicationId = @appId AND u.name = @name", c);
-                cmd.Parameters.AddWithValue("@appId", applicationId);
-                cmd.Parameters.AddWithValue("@name", username);
-                MySqlDataAdapter da = new MySqlDataAdapter(cmd);
-                DataTable dt = new DataTable();
-                da.Fill(dt);
 
-                if (dt.Rows.Count > 0)
-                    DecodeProfileData(dt.Rows[0], values);
-                return values;
+                using (MySqlConnection c = new MySqlConnection(connectionString))
+                {
+                    c.Open();
+                    MySqlCommand cmd = new MySqlCommand(
+                        @"SELECT * FROM my_aspnet_Profiles p
+                    JOIN my_aspnet_Users u ON u.id = p.userId
+                    WHERE u.applicationId = @appId AND u.name = @name", c);
+                    cmd.Parameters.AddWithValue("@appId", app.FetchId(c));
+                    cmd.Parameters.AddWithValue("@name", username);
+                    MySqlDataAdapter da = new MySqlDataAdapter(cmd);
+                    DataTable dt = new DataTable();
+                    da.Fill(dt);
+
+                    if (dt.Rows.Count > 0)
+                        DecodeProfileData(dt.Rows[0], values);
+                    return values;
+                }
             }
             catch (Exception ex)
             {
@@ -437,6 +439,11 @@ namespace MySql.Web.Profile
             }
         }
 
+        /// <summary>
+        /// Sets the values of the specified group of property settings.
+        /// </summary>
+        /// <param name="context">A <see cref="T:System.Configuration.SettingsContext"/> describing the current application usage.</param>
+        /// <param name="collection">A <see cref="T:System.Configuration.SettingsPropertyValueCollection"/> representing the group of property settings to set.</param>
         public override void SetPropertyValues(
             SettingsContext context, SettingsPropertyValueCollection collection)
         {
@@ -461,13 +468,9 @@ namespace MySql.Web.Profile
                     {
                         connection.Open();
 
-                        // create or fetch a new application id
-                        SchemaManager.CreateOrFetchApplicationId(applicationName,
-                            ref applicationId, base.Description, connection);
-
                         // either create a new user or fetch the existing user id
                         int userId = SchemaManager.CreateOrFetchUserId(connection, username, 
-                            applicationId, isAuthenticated);
+                            app.EnsureId(connection), isAuthenticated);
 
                         MySqlCommand cmd = new MySqlCommand(
                             @"INSERT INTO my_aspnet_Profiles  
@@ -480,7 +483,7 @@ namespace MySql.Web.Profile
                         cmd.Parameters.AddWithValue("@stringData", stringData);
                         cmd.Parameters.AddWithValue("@binaryData", binaryData);
                         count = cmd.ExecuteNonQuery();
-                        if (count != 1)
+                        if (count == 0)
                             throw new Exception(Resources.ProfileUpdateFailed);
                         ts.Complete();
                     }
@@ -611,11 +614,12 @@ namespace MySql.Web.Profile
                 c.Open();
 
                 MySqlCommand cmd = new MySqlCommand(
-                @"SELECT p.*, LENGTH(p.stringdata) + LENGTH(p.binarydata) AS profilesize, 
-                u.name FROM my_aspnet_Profiles p 
+                @"SELECT p.*, u.name, u.isAnonymous, u.lastActivityDate,
+                LENGTH(p.stringdata) + LENGTH(p.binarydata) AS profilesize
+                FROM my_aspnet_Profiles p 
                 JOIN my_aspnet_Users u ON u.id = p.userId 
                 WHERE u.applicationId = @appId", c);
-                cmd.Parameters.AddWithValue("@appId", applicationId);
+                cmd.Parameters.AddWithValue("@appId", app.FetchId(c));
 
                 if (usernameToMatch != null)
                 {
@@ -643,13 +647,13 @@ namespace MySql.Web.Profile
                             reader.GetString("name"),
                             reader.GetBoolean("isAnonymous"),
                             reader.GetDateTime("lastActivityDate"),
-                            reader.GetDateTime("lastUpdatdDate"),
+                            reader.GetDateTime("lastUpdatedDate"),
                             reader.GetInt32("profilesize"));
                         pic.Add(pi);
                     }
                 }
                 cmd.CommandText = "SELECT FOUND_ROWS()";
-                totalRecords = (int)cmd.ExecuteScalar();
+                totalRecords = Convert.ToInt32(cmd.ExecuteScalar());
                 return pic;
             }
         }
