@@ -1,4 +1,4 @@
-// Copyright (C) 2007 MySQL AB
+// Copyright (c) 2004-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License version 2 as published by
@@ -29,6 +29,7 @@ using System.Data;
 using System;
 using System.Configuration.Provider;
 using MySql.Web.Security;
+using MySql.Data.MySqlClient;
 
 namespace MySql.Web.Tests
 {
@@ -223,13 +224,15 @@ namespace MySql.Web.Tests
         {
             try
             {
-				Membership.CreateUser("foo", "barbar!");
+                MembershipCreateStatus status;
+                Membership.CreateUser("foo", "barbar!", null, "question", "answer", true, out status);
                 int records;
                 MembershipUserCollection users = Membership.FindUsersByName("F%", 0, 10, out records);
                 Assert.AreEqual(1, records);
                 Assert.AreEqual("foo", users["foo"].UserName);
 
-                Membership.CreateUser("test", "barbar!", "myemail@host.com");
+                Membership.CreateUser("test", "barbar!", "myemail@host.com", 
+                    "question", "answer", true, out status);
                 users = Membership.FindUsersByName("T%", 0, 10, out records);
                 Assert.AreEqual(1, records);
                 Assert.AreEqual("test", users["test"].UserName);
@@ -246,8 +249,9 @@ namespace MySql.Web.Tests
             int numOnline = Membership.GetNumberOfUsersOnline();
             Assert.AreEqual(0, numOnline);
 
-            Membership.CreateUser("foo", "barbar!");
-            Membership.CreateUser("foo2", "barbar!");
+            MembershipCreateStatus status;
+            Membership.CreateUser("foo", "barbar!", null, "question", "answer", true, out status);
+            Membership.CreateUser("foo2", "barbar!", null, "question", "answer", true, out status);
 
             numOnline = Membership.GetNumberOfUsersOnline();
             Assert.AreEqual(2, numOnline);
@@ -256,7 +260,8 @@ namespace MySql.Web.Tests
         [Test]
         public void UnlockUser()
         {
-            Membership.CreateUser("foo", "barbar!");
+            MembershipCreateStatus status;
+            Membership.CreateUser("foo", "barbar!", null, "question", "answer", true, out status);
             Assert.IsFalse(Membership.ValidateUser("foo", "bar2"));
             Assert.IsFalse(Membership.ValidateUser("foo", "bar3"));
             Assert.IsFalse(Membership.ValidateUser("foo", "bar3"));
@@ -279,7 +284,8 @@ namespace MySql.Web.Tests
         [Test]
         public void GetUsernameByEmail()
         {
-            Membership.CreateUser("foo", "barbar!", "foo@bar.com");
+            MembershipCreateStatus status;
+            Membership.CreateUser("foo", "barbar!", "foo@bar.com", "question", "answer", true, out status);
             string username = Membership.GetUserNameByEmail("foo@bar.com");
             Assert.AreEqual("foo", username);
 
@@ -354,9 +360,11 @@ namespace MySql.Web.Tests
         [Test]
         public void GetAllUsers()
         {
+            MembershipCreateStatus status;
             // first create a bunch of users
             for (int i=0; i < 100; i++)
-                Membership.CreateUser(String.Format("foo{0}", i), "barbar!");
+                Membership.CreateUser(String.Format("foo{0}", i), "barbar!", null, 
+                    "question", "answer", true, out status);
 
             MembershipUserCollection users = Membership.GetAllUsers();
             Assert.AreEqual(100, users.Count);
@@ -394,6 +402,11 @@ namespace MySql.Web.Tests
                     Assert.Fail("This should have thrown an exception");
                 Assert.AreEqual("barbar!", password);
             }
+            catch (MembershipPasswordException)
+            {
+                if (requireQA && answer != null)
+                    Assert.Fail("This should not have thrown an exception");
+            }
             catch (ProviderException)
             {
                 if (requireQA && answer != null)
@@ -410,10 +423,47 @@ namespace MySql.Web.Tests
             GetPasswordHelper(true, true, "blue");
         }
 
+        /// <summary>
+        /// Bug #38939 MembershipUser.GetPassword(string answer) fails when incorrect answer is passed.
+        /// </summary>
+        [Test]
+        public void GetPasswordWithWrongAnswer()
+        {
+            MembershipCreateStatus status;
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("requiresQuestionAndAnswer", "true");
+            config.Add("enablePasswordRetrieval", "true");
+            config.Add("passwordFormat", "Encrypted");
+            config.Add("applicationName", "/");
+            provider.Initialize(null, config);
+            provider.CreateUser("foo", "barbar!", "foo@bar.com", "color", "blue", true, null, out status);
+
+            MySQLMembershipProvider provider2 = new MySQLMembershipProvider();
+            NameValueCollection config2 = new NameValueCollection();
+            config2.Add("connectionStringName", "LocalMySqlServer");
+            config2.Add("requiresQuestionAndAnswer", "true");
+            config2.Add("enablePasswordRetrieval", "true");
+            config2.Add("passwordFormat", "Encrypted");
+            config2.Add("applicationName", "/");
+            provider2.Initialize(null, config2);
+
+            try
+            {
+                string pw = provider2.GetPassword("foo", "wrong");
+                Assert.Fail("Should have  failed");
+            }
+            catch (MembershipPasswordException)
+            {
+            }
+        }
+
         [Test]
         public void GetUser()
         {
-            Membership.CreateUser("foo", "barbar!");
+            MembershipCreateStatus status;
+            Membership.CreateUser("foo", "barbar!", null, "question", "answer", true, out status);
             MembershipUser user = Membership.GetUser(1);
             Assert.AreEqual("foo", user.UserName);
 
@@ -443,12 +493,16 @@ namespace MySql.Web.Tests
         [Test]
         public void FindUsers()
         {
+            MembershipCreateStatus status;
             for (int i=0; i < 100; i++)
-                Membership.CreateUser(String.Format("boo{0}", i), "barbar!");
+                Membership.CreateUser(String.Format("boo{0}", i), "barbar!", null, 
+                    "question", "answer", true, out status);
             for (int i=0; i < 100; i++)
-                Membership.CreateUser(String.Format("foo{0}", i), "barbar!");
+                Membership.CreateUser(String.Format("foo{0}", i), "barbar!", null, 
+                    "question", "answer", true, out status);
             for (int i=0; i < 100; i++)
-                Membership.CreateUser(String.Format("schmoo{0}", i), "barbar!");
+                Membership.CreateUser(String.Format("schmoo{0}", i), "barbar!", null, 
+                    "question", "answer", true, out status);
 
 
             MembershipUserCollection users = Membership.FindUsersByName("fo%");
@@ -463,5 +517,151 @@ namespace MySql.Web.Tests
                 Assert.AreEqual(String.Format("foo2{0}", index++), user.UserName);
 
         }
+
+        [Test]
+        public void CreateUserWithNoQA()
+        {
+            MembershipCreateStatus status;
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("requiresQuestionAndAnswer", "true");
+            config.Add("passwordFormat", "clear");
+            config.Add("applicationName", "/");
+            provider.Initialize(null, config);
+
+            try
+            {
+                provider.CreateUser("foo", "barbar!", "foo@bar.com", "color", null, true, null, out status);
+                Assert.Fail();
+            }
+            catch (Exception ex)
+            {
+                Assert.IsTrue(ex.Message.StartsWith("Password answer supplied is invalid"));
+            }
+            try
+            {
+                provider.CreateUser("foo", "barbar!", "foo@bar.com", "", "blue", true, null, out status);
+                Assert.Fail();
+            }
+            catch (Exception ex)
+            {
+                Assert.IsTrue(ex.Message.StartsWith("Password question supplied is invalid"));
+            }
+        }
+
+        [Test]
+        public void MinRequiredAlpha()
+        {
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("applicationName", "/");
+            config.Add("minRequiredNonalphanumericCharacters", "3");
+            provider.Initialize(null, config);
+
+            MembershipCreateStatus status;
+            MembershipUser user = provider.CreateUser("foo", "pw!pass", "email", null, null, true, null, out status);
+            Assert.IsNull(user);
+
+            user = provider.CreateUser("foo", "pw!pa!!", "email", null, null, true, null, out status);
+            Assert.IsNotNull(user);
+        }
+
+        /// <summary>
+        /// Bug #35332 GetPassword() don't working (when PasswordAnswer is NULL) 
+        /// </summary>
+        [Test]
+        public void GetPasswordWithNullValues()
+        {
+            MembershipCreateStatus status;
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("requiresQuestionAndAnswer", "false");
+            config.Add("enablePasswordRetrieval", "true");
+            config.Add("passwordFormat", "clear");
+            config.Add("applicationName", "/");
+            provider.Initialize(null, config);
+
+            MembershipUser user = provider.CreateUser("foo", "barbar!", "foo@bar.com", null, null, true, null, out status);
+            Assert.IsNotNull(user);
+
+            string pw = provider.GetPassword("foo", null);
+            Assert.AreEqual("barbar!", pw);
+        }
+
+        /// <summary>
+        /// Bug #35336 GetPassword() return wrong password (when format is encrypted) 
+        /// </summary>
+        [Test]
+        public void GetEncryptedPassword()
+        {
+            MembershipCreateStatus status;
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("requiresQuestionAndAnswer", "false");
+            config.Add("enablePasswordRetrieval", "true");
+            config.Add("passwordFormat", "encrypted");
+            config.Add("applicationName", "/");
+            provider.Initialize(null, config);
+
+            MembershipUser user = provider.CreateUser("foo", "barbar!", "foo@bar.com", null, null, true, null, out status);
+            Assert.IsNotNull(user);
+
+            string pw = provider.GetPassword("foo", null);
+            Assert.AreEqual("barbar!", pw);
+        }
+
+        /// <summary>
+        /// Bug #42574	ValidateUser does not use the application id, allowing cross application login
+        /// </summary>
+        [Test]
+        public void CrossAppLogin()
+        {
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("applicationName", "/");
+            config.Add("passwordStrengthRegularExpression", "bar.*");
+            config.Add("passwordFormat", "Clear");
+            provider.Initialize(null, config);
+            MembershipCreateStatus status;
+            provider.CreateUser("foo", "bar!bar", null, null, null, true, null, out status);
+
+            MySQLMembershipProvider provider2 = new MySQLMembershipProvider();
+            NameValueCollection config2 = new NameValueCollection();
+            config2.Add("connectionStringName", "LocalMySqlServer");
+            config2.Add("applicationName", "/myapp");
+            config2.Add("passwordStrengthRegularExpression", ".*");
+            config2.Add("passwordFormat", "Clear");
+            provider2.Initialize(null, config2);
+
+            bool worked = provider2.ValidateUser("foo", "bar!bar");
+            Assert.AreEqual(false, worked);
+        }
+
+        /// <summary>
+        /// Bug #41408	PasswordReset not possible when requiresQuestionAndAnswer="false"
+        /// </summary>
+        [Test]
+        public void ResetPassword()
+        {
+            provider = new MySQLMembershipProvider();
+            NameValueCollection config = new NameValueCollection();
+            config.Add("connectionStringName", "LocalMySqlServer");
+            config.Add("applicationName", "/");
+            config.Add("passwordStrengthRegularExpression", "bar.*");
+            config.Add("passwordFormat", "Clear");
+            config.Add("requiresQuestionAndAnswer", "false");
+            provider.Initialize(null, config);
+
+            MembershipCreateStatus status;
+            provider.CreateUser("foo", "bar!bar", null, null, null, true, null, out status);
+
+            MembershipUser u = provider.GetUser("foo", false);
+            string newpw = provider.ResetPassword("foo", null);
+        }
     }
 }
